function [center, U, obj_fcn] = fuzzy_c_means_CR(data, no_clusters, info, store_trace_centers)
% Improved Fuzzy C-Means which uses recurrent confidence coefficients.
% Methods was described / detailed in the PhD thesis M.Zbancioc - New
% methods of pattern descripsion of mono si bi-dimensional signals in order
% to improve the performance of automatic recogniotion", 2009 and in 2 
% papers presented at international conferences.
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

if nargin<3, info = 0; end
if nargin<4, trace_centers=0; end

[data_len, data_dim] = size(data);
if data_len < data_dim, data = data'; [data_len, data_dim] = size(data); end

expo = 2;           % Exponent used for U matrix (membership degree)
max_iter = 100;		% Maximum number of itteration
min_impro = 1e-5;	% Min. improvement
min_impro2 = 1e-2;

U = rand(no_clusters, data_len);    % Initialization of fuzzy partitions
col_sum = sum(U);
U = U./col_sum(ones(no_clusters, 1), :); 

% Objectiv function (will be keeped/ used for all itteration)
obj_fcn = zeros(max_iter, 1);
trace_centers = zeros(max_iter, no_clusters, data_dim);

U2 = ones(size(U)); start_recurent =0;
% Main loop
for i = 1:max_iter,
	mf = U.^expo;       % MF matrix after exponential modification
    center = mf*data./((ones(data_dim, 1)*sum(mf'))'); % cluster centres 
    trace_centers(i,:,:) = center;
    
    dist = zeros(no_clusters, data_len); %matrix of distances to each center

    if data_dim > 1,
        for k = 1:no_clusters
        	dist(k, :) = sqrt(sum(((data - ones(data_len, 1)*center(k, :)).^2)'));
        end
    else	% 1-D data
        for k = 1:no_clusters,
            dist(k, :) = abs(center(k)-data)';
        end
    end
    
    obj_fcn(i) = sum(sum((dist.^2).*mf));  % objective function
    tmp = dist.^(-2/(expo-1));             % calculate new U, suppose expo != 1
    U = tmp./(ones(no_clusters, 1)*sum(tmp)); % new values of partition matrix

	if info, 
		fprintf('Iteration count = %d, obj. fcn = %f\n', i, obj_fcn(i));
	end
	% check termination condition
	if i > 1,
		if abs(obj_fcn(i) - obj_fcn(i-1)) < min_impro, break; end
        if abs(obj_fcn(i) - obj_fcn(i-1)) < min_impro2,
            start_recurent =1;
        end
%         for j=1:size(U,1)
%             for k=1:size(U,2)
%                 U2(j,k) = Sugeno_2i1o(abs(obj_fcn(i) - obj_fcn(i-1)), U(j,k));
%             end
%         end
%         U=U.*U2;
%         U = U./(ones(no_clusters, 1)*sum(U));
%%------------------------------------------------
%         if abs(obj_fcn(i) - obj_fcn(i-1))<0.001,
%             U2 = zeros(size(U));
%             for j=1:size(U,1)
%                 for k=1:size(U,2)
%                     U2(j,k) = Sugeno_1i1o(U(j,k));
%                 end
%             end
%             U=U.*U2;
%             U = U./(ones(no_clusters, 1)*sum(U));
%         end
        if start_recurent==1,
            U2=U2.*(2*U.^2.*(U<0.5) + (1-2*(1-U).^2).*(U>=0.5));
        end            
        U=U2.*U;
	end
end

iter_n = i;	% Actual number of iterations 
obj_fcn(iter_n+1:max_iter) = [];
trace_centers(iter_n+1:max_iter,:,:) = [];
if store_trace_centers ==1, center = trace_centers; end

%--------------------------------------------------------------------------
function out = Sugeno_2i1o(in1, in2)
%membership functions for the first input variable
mfx1 = [1e-5 0.001 0.01; 0.01 0.1 1; 0.1 1 100];
mfy1 = [1        1    0;    0   1 0;   0 1   1];
%membership functions for the second input variable
mfx2 = [0 0.1 0.25 0.3; 0.1 0.25  0.65 0.8; 0.65 0.8 0.9 1];
mfy2 = [1   1    0   0;   0    1     1   0;    0   1   1 1];
%value of the output singletons
out = [0.1 0.3 1; 0.2 0.4 0.8; 0.4 0.5 0.6];

miu1 = zeros(size(mfx1,1),1);
for i = 1: size(mfx1,2)-1;
    m = (in1-mfx1(:,i))./(mfx1(:,i+1)-mfx1(:,i)) .* (mfy1(:,i)<mfy1(:,i+1))+...
              (mfx1(:,i+1)-in1)./(mfx1(:,i+1)-mfx1(:,i)) .* (mfy1(:,i)>mfy1(:,i+1));
    m = m.*abs(mfy1(:,i)-mfy1(:,i+1)) + min(mfy1(:,i),mfy1(:,i+1));
    m = m.*(in1>=mfx1(:,i) & in1<mfx1(:,i+1));
    miu1 = miu1+m;
end

miu2 = zeros(size(mfx2,1),1);
for i = 1: size(mfx2,2)-1;
    m = (in2-mfx2(:,i))./(mfx2(:,i+1)-mfx2(:,i)) .* (mfy2(:,i)<mfy2(:,i+1))+...
              (mfx2(:,i+1)-in2)./(mfx2(:,i+1)-mfx2(:,i)) .* (mfy2(:,i)>mfy2(:,i+1));
    m = m.*abs(mfy2(:,i)-mfy2(:,i+1)) + min(mfy2(:,i),mfy2(:,i+1));
    m = m.*(in2>=mfx2(:,i) & in2<mfx2(:,i+1));
    miu2 = miu2+m;
end
s1=0; s2=0;
for i = 1: size(mfx1,1)
    for j = 1 : size(mfx2,1)
        s1 = s1 + out(i,j)*min(miu1(i), miu2(j));
        s2 = s2 + min(miu1(i), miu2(j));
    end
end
out = s1/s2;
%--------------------------------------------------------------------------
function out = Sugeno_1i1o(in)
%membership functions for the input variable
mfx = [0 0.1 0.25 0.3; 0.1 0.25  0.65 0.8; 0.65 0.8 0.9 1];
mfy = [1   1    0   0;   0    1     1   0;    0   1   1 1];
%value of the output singletons
out = [0.1 0.3 1];

miu = zeros(size(mfx,1),1);
for i = 1: size(mfx,2)-1;
    m = (in-mfx(:,i))./(mfx(:,i+1)-mfx(:,i)) .* (mfy(:,i)<mfy(:,i+1))+...
              (mfx(:,i+1)-in)./(mfx(:,i+1)-mfx(:,i)) .* (mfy(:,i)>=mfy(:,i+1));
    m = m.*abs(mfy(:,i)-mfy(:,i+1)) + min(mfy(:,i),mfy(:,i+1));
    m = m.*(in>=mfx(:,i) & in<mfx(:,i+1));
    miu = miu+m;
end
out = out*miu;